@@ -38,111 +38,93 @@ def load_data():
     return pd.read_parquet(LOCAL_DATA_PATH)
 
 
-def bubble_diffuse(x, h, b_lim, diffuse_dust_rho, bubble):
+def bubble_diffuse(x,h,b_lim,diffuse_dust_rho,bubble): 
     span = 0.01
     span_0 = h / np.sin(np.deg2rad(np.abs(b_lim)))
     Cum_EBV_0 = span_0 * diffuse_dust_rho
-    C_0 = Cum_EBV_0 * (1 - np.exp(-bubble / span_0))
-
-    exp_term = np.exp(5 * bubble / span)
-    a = 1 / exp_term
-    b = 1 / (1 + exp_term)
+    C_0 = Cum_EBV_0 * (1 - np.exp(- (bubble) / span_0))
+    f = (Cum_EBV_0 * (1 - np.exp(-x / span_0))) - C_0
+    exp_n = np.exp(5 * bubble /span)
+    a = 1 / exp_n
+    b = 1 / (1 + exp_n)
     c = 0.5
-    deta = C_0 / ((1 + a) * (c - b))
+    deta = C_0/((1+a)*(c-b))
+    return np.where(x < (bubble), 0, f) + deta*(1+a)*((1 / (1 + np.exp(-5 * ((x - bubble)/span))) )-b)
 
-    res = np.zeros_like(x)
-    mask = x >= bubble
-    res[mask] = (Cum_EBV_0 * (1 - np.exp(-x[mask] / span_0)) - C_0)[mask]
-    res += deta * (1 + a) * ((1 / (1 + np.exp(-5 * ((x - bubble) / span)))) - b)
-    return res
+def component4(x, b_lim, bubble, diffuse_dust_rho, h, distance_1, span_1, Cum_EBV_1, distance_2, span_2, Cum_EBV_2, distance_3, span_3, Cum_EBV_3, distance_4, span_4, Cum_EBV_4):
+    Numerator_1 = Cum_EBV_1*(1/np.exp(5 * (distance_1 + (span_1*2) + bubble) /span_1) + 1)
+    Numerator_2 = Cum_EBV_2*(1/np.exp(5 * (distance_2 + (span_2*2) + bubble)/span_2) + 1)
+    Numerator_3 = Cum_EBV_3*(1/np.exp(5 * (distance_3 + (span_3*2) + bubble)/span_3) + 1)
+    Numerator_4 = Cum_EBV_4*(1/np.exp(5 * (distance_4 + (span_4*2) + bubble)/span_4) + 1)
+    
+    return (bubble_diffuse(x,h,b_lim,diffuse_dust_rho,bubble)
+                     
+                    +((Numerator_1/ (1 + np.exp(-5 * ((x) - (distance_1 + (span_1*2) + bubble))/span_1))) 
+                    -(Numerator_1 / (1 + np.exp(5 * (distance_1 + (span_1*2) + bubble)/span_1))))
+                    
+                    +((Numerator_2 / (1 + np.exp(-5 * ((x) - (distance_2 + (span_2*2) + bubble))/span_2))) 
+                    -(Numerator_2 / (1 + np.exp(5 * ((distance_2 + (span_2*2) + bubble))/span_2))))
 
+                    +((Numerator_3 / (1 + np.exp(-5 * ((x) - (distance_3 + (span_3*2) + bubble))/span_3))) 
+                    -(Numerator_3 / (1 + np.exp(5 * ((distance_3 + (span_3*2) + bubble))/span_3))))
 
-def component4(x, b_lim, bubble, diffuse_dust_rho, h,
-               d1, s1, c1, d2, s2, c2, d3, s3, c3, d4, s4, c4):
-    def sigmoid_contrib(d, s, c):
-        term = d + 2 * s + bubble
-        numerator = c * (1 / np.exp(5 * term / s) + 1)
-        return numerator, term, s
-
-    terms = [sigmoid_contrib(d, s, c) for d, s, c in [(d1, s1, c1), (d2, s2, c2), (d3, s3, c3), (d4, s4, c4)]]
-
-    result = bubble_diffuse(x, h, b_lim, diffuse_dust_rho, bubble)
-    for numerator, term, s in terms:
-        result += numerator / (1 + np.exp(-5 * (x - term) / s)) - numerator / (1 + np.exp(5 * term / s))
-    return result
-
-
-def diffusion_derived_function(x, b_lim, diffuse_dust_rho, h):
+                    +((Numerator_4 / (1 + np.exp(-5 * ((x) - (distance_4 + (span_4*2) + bubble))/span_4))) 
+                    -(Numerator_4 / (1 + np.exp(5 * ((distance_4 + (span_4*2) + bubble))/span_4))))
+                    )       
+ 
+def diffusion_derived_function(x, b_lim, diffuse_dust_rho, h ):
     span_0 = h / np.sin(np.deg2rad(np.abs(b_lim)))
-    return diffuse_dust_rho * np.exp(-x / span_0)
-
+    return diffuse_dust_rho * (np.exp(- x / span_0))
 
 def sigmoid(x, a, b, c):
     return c / (1 + np.exp(-b * (x - a)))
 
-
 def derivative_of_sigmoid(x, a, b, c):
-    sig = sigmoid(x, a, b, 1)
-    return b * c * sig * (1 - sig)
-
+    return b * c * sigmoid(x, a, b, 1) * (1 - (sigmoid(x, a, b, 1)))
 
 def sigmoid_of_component(bubble, distance, span, Cum_EBV):
-    a = distance + 2 * span + bubble
+    a = distance + (2*span) + bubble
     b = 5 / span
-    c = Cum_EBV * (1 / np.exp(5 * a / span) + 1)
+    c = Cum_EBV*(1/np.exp(5 * a /span) + 1)
     return a, b, c
 
-
-def derivative_of_component4(x, b_lim, bubble, diffuse_dust_rho, h,
-                             d1, s1, c1, d2, s2, c2, d3, s3, c3, d4, s4, c4):
-    a1, b1, c1 = sigmoid_of_component(bubble, d1, s1, c1)
-    a2, b2, c2 = sigmoid_of_component(bubble, d2, s2, c2)
-    a3, b3, c3 = sigmoid_of_component(bubble, d3, s3, c3)
-    a4, b4, c4 = sigmoid_of_component(bubble, d4, s4, c4)
-
-    base = np.zeros_like(x)
-    mask = x >= bubble
-    base[mask] = diffusion_derived_function(x[mask], b_lim[mask], diffuse_dust_rho[mask], h[mask])
-
-    return base + \
-        derivative_of_sigmoid(x, a1, b1, c1) + \
-        derivative_of_sigmoid(x, a2, b2, c2) + \
-        derivative_of_sigmoid(x, a3, b3, c3) + \
-        derivative_of_sigmoid(x, a4, b4, c4)
-
+def derivative_of_component4(x, b_lim, bubble, diffuse_dust_rho, h, distance_1, span_1, Cum_EBV_1, distance_2, span_2, Cum_EBV_2, distance_3, span_3, Cum_EBV_3, distance_4, span_4, Cum_EBV_4):
+    a_1, b_1, c_1 = sigmoid_of_component(bubble, distance_1, span_1, Cum_EBV_1)
+    a_2, b_2, c_2 = sigmoid_of_component(bubble, distance_2, span_2, Cum_EBV_2)
+    a_3, b_3, c_3 = sigmoid_of_component(bubble, distance_3, span_3, Cum_EBV_3)
+    a_4, b_4, c_4 = sigmoid_of_component(bubble, distance_4, span_4, Cum_EBV_4)
+    return (np.where(x < bubble, 0, diffusion_derived_function(x, b_lim, diffuse_dust_rho, h)) 
+            + derivative_of_sigmoid(x, a_1, b_1, c_1) 
+            + derivative_of_sigmoid(x, a_2, b_2, c_2) 
+            + derivative_of_sigmoid(x, a_3, b_3, c_3) 
+            + derivative_of_sigmoid(x, a_4, b_4, c_4) 
+            )
 
 def map(df):
     distance = df['distance'].fillna(df['max_distance'])
-    EBV = component4(distance, df['b_lim'], df['bubble'], df['diffuse_dust_rho'], df['h'],
-                     df['distance_1'], df['span_1'], df['Cum_EBV_1'],
-                     df['distance_2'], df['span_2'], df['Cum_EBV_2'],
-                     df['distance_3'], df['span_3'], df['Cum_EBV_3'],
-                     df['distance_4'], df['span_4'], df['Cum_EBV_4'])
-
-    dust = derivative_of_component4(distance, df['b_lim'], df['bubble'], df['diffuse_dust_rho'], df['h'],
-                                    df['distance_1'], df['span_1'], df['Cum_EBV_1'],
-                                    df['distance_2'], df['span_2'], df['Cum_EBV_2'],
-                                    df['distance_3'], df['span_3'], df['Cum_EBV_3'],
-                                    df['distance_4'], df['span_4'], df['Cum_EBV_4'])
-
+    EBV = component4(distance, df['b_lim'], df['bubble'], df['diffuse_dust_rho'], df['h'], 
+                    df['distance_1'], df['span_1'], df['Cum_EBV_1'], 
+                    df['distance_2'], df['span_2'], df['Cum_EBV_2'],
+                    df['distance_3'], df['span_3'], df['Cum_EBV_3'],
+                    df['distance_4'], df['span_4'], df['Cum_EBV_4'])
+    dust = derivative_of_component4(distance, df['b_lim'], df['bubble'], df['diffuse_dust_rho'], df['h'], 
+                    df['distance_1'], df['span_1'], df['Cum_EBV_1'], 
+                    df['distance_2'], df['span_2'], df['Cum_EBV_2'],
+                    df['distance_3'], df['span_3'], df['Cum_EBV_3'],
+                    df['distance_4'], df['span_4'], df['Cum_EBV_4']) 
     sigma_finally = np.empty_like(df['sigma'], dtype=float)
-
-    mask1 = distance < 1
-    sigma_finally[mask1] = np.nanmin([df['sigma'][mask1], df['sigma_0_2'][mask1]], axis=0)
-
-    mask2 = (distance >= 1) & (distance < 2)
-    sigma_finally[mask2] = np.nanmin([df['sigma'][mask2], df['sigma_0_2'][mask2], df['sigma_1_4'][mask2]], axis=0)
-
-    mask3 = (distance >= 2) & (distance < 4)
-    sigma_finally[mask3] = np.nanmin([df['sigma_1_4'][mask3], df['sigma_2_max'][mask3]], axis=0)
-
-    sigma_finally[distance >= 4] = df['sigma_2_max'][distance >= 4]
-
+    mask = distance < 1
+    sigma_finally[mask] = np.nanmin(np.array([df['sigma'][mask], df['sigma_0_2'][mask]]), axis=0)
+    mask = (distance >= 1) & (distance < 2)
+    sigma_finally[mask] = np.nanmin(np.array([df['sigma'][mask], df['sigma_0_2'][mask], df['sigma_1_4'][mask]]), axis=0)
+    mask = (distance >= 2) & (distance < 4)
+    sigma_finally[mask] = np.nanmin(np.array([df['sigma_1_4'][mask], df['sigma_2_max'][mask]]), axis=0)
+    mask = distance >= 4
+    sigma_finally[mask] = df['sigma_2_max'][mask]
     return EBV, dust, sigma_finally, df['max_distance']
 
 
 
-
 def _dustmaps3d_worker(args):
     l_chunk, b_chunk, d_chunk = args
     results = [dustmaps3d(l, b, d) for l, b, d in zip(l_chunk, b_chunk, d_chunk)]
